"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
/** Insert a middleware into this route */
function Middleware(middleware) {
    return function MiddlewareDecorator(target, key) {
        const func = key ? target[key] : target;
        const middlewares = func.middlewares || (func.middlewares = []);
        /**
         * Ensure middlewares are executed from the top to the bottom.
         * Note that this is different from the execution order of the decorators.
         * Decorators executes from the bottom to the top.
         * However, when it comes to middlewares, it is very wired.
         *
         * For example, if we use the ordering of decorator directly:
         *
         * @Get('/')            // execution order
         * @Middleware1         // 3
         * @Middleware2         // 2
         * @Middleware3         // 1
         * async route () {}    // 4
         *
         * And semantically middlewares should be executed before the final handler.
         * So that is why it is ordered like this.
         */
        middlewares.unshift(middleware);
    };
}
exports.Middleware = Middleware;
/** Create a middleware decorator */
exports.createDecorator = Middleware;
//# sourceMappingURL=middleware.js.map