"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const decorators = require("./route");
const inflection = require("inflection");
const def_1 = require("./def");
function ResourceDecoratorFactory(prefix = '/', name = '') {
    return function ResourceDecorator(target) {
        if (name == null) {
            // set name to null to disable auto name prefix
            name = '';
        }
        else if (!name) {
            // Auto-detect resource name from the controller's name
            const splitName = inflection.underscore(target.name).split('_');
            const lastWord = splitName[splitName.length - 1];
            if (def_1.LAST_WORD_BLACKLIST.includes(lastWord)) {
                splitName.splice(-1, 1);
            }
            name = splitName.join('_');
        }
        for (const key in def_1.REST_MAP) {
            if (target.prototype[key] == null)
                continue;
            const opts = def_1.REST_MAP[key];
            let formatedName = '';
            // Give the route a name
            // egg-core@4.4.0/lib/utils/router.js:162
            if (name) {
                if (opts.member) {
                    formatedName = inflection.singularize(name);
                }
                else {
                    formatedName = inflection.pluralize(name);
                }
                if (opts.namePrefix) {
                    formatedName = opts.namePrefix + formatedName;
                }
            }
            // Skip if user defines the route manually
            const routeTable = target.prototype[def_1.ROUTE_TABLE];
            if (routeTable && routeTable.find(entry => entry.key === key)) {
                continue;
            }
            const methods = Array.isArray(opts.method) ? opts.method : [opts.method];
            for (const method of methods) {
                // Use the existing decorators to register routes
                decorators[method](opts.suffix, formatedName)(target.prototype, key);
            }
        }
        // Decorate target with @Routes
        const namePrefix = name && inflection.pluralize(name.split(/\s+/).map(x => inflection.underscore(x)).map(inflection.dasherize).join('-'));
        const finalPrefix = path.join(prefix, namePrefix);
        decorators.Routes(finalPrefix)(target);
    };
}
function Resource(...args) {
    if (typeof args[0] === 'function') {
        ResourceDecoratorFactory()(args[0]);
    }
    else {
        return ResourceDecoratorFactory(...args);
    }
}
exports.Resource = Resource;
//# sourceMappingURL=resource.js.map