"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const _ = require("lodash");
const path = require("path");
const store_1 = require("./store");
const def_1 = require("./def");
/** Create decorators like @Get, @Post, etc. */
function generateRouteDecorator(method) {
    return function RouteDecoratorFactory(path, name) {
        return function RouteDecorator(target, key) {
            // All routes will be stored in the route table
            // at class.prototype[ROUTE_TABLE]
            const routeTable = target[def_1.ROUTE_TABLE] || (target[def_1.ROUTE_TABLE] = []);
            // Inject an array at method.middlewares for easier middlewares' modification
            const middlewares = target[key].middlewares || (target[key].middlewares = []);
            routeTable.push({ key, name, path, method, middlewares });
        };
    };
}
/** Underlying implementation of @Routes */
function RoutesDecoratorFactory(prefix = '/') {
    return function RoutesDecorator(target) {
        const { app } = store_1.store;
        const routeTable = target.prototype[def_1.ROUTE_TABLE] || [];
        // Register all routes in the route table into the app before it starts
        app.beforeStart(() => {
            for (const entry of routeTable) {
                // Get the original middleware from egg
                const baseMiddleware = _.get(app, target.prototype.pathName)[entry.key];
                // Combine all middlewares from the route itself and the controller
                const localMiddlewares = entry.middlewares;
                const commonMiddlewares = target.middlewares || [];
                const args = [
                    path.posix.join('/', prefix, entry.path),
                    ...commonMiddlewares,
                    ...localMiddlewares,
                    baseMiddleware
                ];
                if (entry.name) {
                    args.unshift(entry.name);
                }
                // @ts-ignore
                // Register route into the app
                app.router[entry.method](...args);
            }
        });
    };
}
function Routes(arg) {
    if (typeof arg === 'string') {
        return RoutesDecoratorFactory(arg);
    }
    else {
        RoutesDecoratorFactory()(arg);
    }
}
exports.Routes = Routes;
exports.Get = generateRouteDecorator('get');
exports.Post = generateRouteDecorator('post');
exports.Put = generateRouteDecorator('put');
exports.Patch = generateRouteDecorator('patch');
exports.Delete = generateRouteDecorator('delete');
//# sourceMappingURL=route.js.map